﻿using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Netmash.Infrastructure.Messaging.Abstractions;
using Netmash.Infrastructure.Messaging.Host.Models;
using System;

namespace Netmash.Infrastructure.Messaging.Host
{
    public static class DependencyInjectionExtensions
    {
        public static IServiceCollection AddMessagingHost(this IServiceCollection serviceCollection, IConfiguration configuration, Action<IMessagingHostBuilder> configure)
        {
            serviceCollection.Configure<MessagingHostOptions>(configuration.GetSection("Messaging").GetSection("Host"));

            serviceCollection.AddSingleton<IMessagingHost, MessagingHost>();
            serviceCollection.AddHostedService<MessagingHostService>();
            serviceCollection.AddSingleton<MessagingContextAccessor>();
            serviceCollection.Decorate<IMessageBusPublisher, MessagingContextBusPublisherDecorator>();


            serviceCollection.TryAddSingleton(serviceCollection);

            var builder = new MessagingHostBuilder(serviceCollection);
            configure.Invoke(builder);

            return serviceCollection;
        }
    }
}
